/*
    Liblist 1.3, a utility to display the needed libraries in a Fargo program.
    Copyright (C) 1997 Tim Gerla <timg@means.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

int main( int argc, char **argv )
{
  FILE *prgm;
  short offset, numlibs;
  int x = 0, i=0;
  char *string;
  char c, libs[10][9]; /* ten libs should be enough */

  union {
    short w;
    char b[2];
  } ofs;

  if( argc != 2 ) {
    printf( "%s: Wrong number of arguments.\n", argv[0] );
    printf( "Syntax: %s <file>\n", argv[0] );
    printf( "Lists the required libraries in a Fargo program.\n" );
    exit(-1);
  }

  if((prgm=fopen( argv[1], "rb" ))== NULL) {
    perror( "fopen" );
    exit(-1);
  }

  string = (char *)malloc(100);
  
  /* 0x58+6 gets us to the place that points to the
     list of libraries, see fargo.txt for more info. */
  fseek( prgm, 0x58+6, SEEK_SET );
  fread( &offset, 1, 2, prgm );
  
  /* Fix the *#&@^@ endian & get past the .92p header */
  ofs.b[1] = offset;
  ofs.b[0] = offset >> 0x08;
  offset = ofs.w+0x5A;
  
  /* This reads the number of libraries from the object code. */
  fseek( prgm, offset-2, SEEK_SET ); 
  fread( string, 1, 2, prgm );
  /* And then fixes the endian. */
  ofs.b[0]=string[1];
  ofs.b[1]=string[0];
  numlibs = ofs.w;

  printf( "Liblist v1.3, (C) 1997 Tim Gerla <timg@means.net>\n" );
  printf( "Released under the GPL. Please see the COPYING file for details.\n" );
  printf( "%d Fargo libraries required for \'%s\':\n\n",numlibs, argv[1] );


  for( x = 0; x < numlibs; x++ ) {
    do {
      c = fgetc(prgm);
      libs[x][i++]=c;
    } while( c!=0 && (i<8));
    i=0;
    printf( "%d. %s\n",x+1,libs[x] );
  }

  /* Clean up. */
  fclose( prgm );
  exit(0);
}

